@@ -1,86 +1,27 @@
+# Default chdir is null and will be set at the first call of the function
+OPTS_SLURM    <- new.env(parent = emptyenv())
+OPTS_R        <- new.env(parent = emptyenv())
+OPTS_PREAMBLE <- new.env(parent = emptyenv())
 
-#' Get and set default options for `sbatch` and `slurmR` internals
-#'
-#' Most of the functions in the `slurmR` package use `tmp_path` and `job-name`
-#' options to write and submit jobs to **Slurm**. These options have global
-#' defaults that are set and retrieved using `opts_slurmR`.
-#'
-#' Whatever the path specified on `tmp_path`, all nodes should have access to it.
-#' Moreover, it is recommended to use a path located in a high-performing drive.
-#' See for example [disk staging](https://en.wikipedia.org/w/index.php?title=Disk_staging&oldid=908353920).
-#'
-#' The `tmp_path` directory is only created at the time that one of the functions
-#' needs to I/O files. Job creation calls like [Slurm_EvalQ] and [Slurm_lapply]
-#' do such.
-#'
-#' @details Current supported options are:
-#'
-#' Debugging mode
-#'
-#' - `debug_on : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$debug_on, "desc")}
-#'
-#' - `debug_off : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$debug_off, "desc")}
-#'
-#' - `get_debug : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_debug, "desc")}
-#'
-#' Verbose mode
-#'
-#' - `verbose_on : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$verbose_on, "desc")}
-#'
-#' - `verbose_off : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$verbose_off, "desc")}
-#'
-#' - `get_verbose : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_verbose, "desc")}
-#'
-#' Slurm options
-#'
-#' - `set_tmp_path : function (path, recursive = TRUE)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$set_tmp_path, "desc")}
-#'
-#' - `get_tmp_path : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_tmp_path, "desc")}
-#'
-#' - `set_job_name : function (path, check = TRUE, overwrite = TRUE)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$set_job_name, "desc")}.
-#'
-#' - `get_job_name : function (check = TRUE)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_job_name, "desc")}
-#'
-#' Other options
-#'
-#' - `get_cmd : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_cmd, "desc")}
-#'
-#'
-#' For general set/retrieve options
-#'
-#' - `set_opts : function (...)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$set_opts, "desc")}
-#'
-#' - `get_opts_job : function (...)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_opts_job, "desc")}
-#'
-#' - `get_opts_r : function (...)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_opts_r, "desc")}
-#'
-#'
-#' @examples
-#'
-#' # Common setup
-#' \dontrun{
-#' opts_slurmR$set_tmp_path("/staging/pdt/vegayon")
-#' opts_slurmR$set_job_name("simulations-1")
-#' opts_slurm$set_opts(partition="thomas", account="lc_pdt")
-#' }
-#'
-#' @export
-opts_slurmR <- (function() {
+.opts_slurmR <- function() {
 
-  # Default chdir is null and will be set at the first call of the function
-  OPTS_SLURM            <- new.env(parent = emptyenv())
-  OPTS_SLURM$`job-name` <- NULL
+  rm(list = ls(envir = OPTS_SLURM, all.names = TRUE), envir = OPTS_SLURM)
+  rm(list = ls(envir = OPTS_R, all.names = TRUE), envir = OPTS_R)
+  rm(list = ls(envir = OPTS_PREAMBLE, all.names = TRUE), envir = OPTS_PREAMBLE)
 
-  OPTS_R          <- new.env(parent = emptyenv())
   OPTS_R$tmp_path <- NULL
+  OPTS_SLURM$`job-name` <- NULL
   OPTS_R$debug    <- FALSE
   OPTS_R$cmd      <- "sbatch"
   OPTS_R$verbose  <- FALSE
-
+  OPTS_PREAMBLE$dat <- NULL
 
   # JOB PATH -------------------------------------------------------------------
   # Function to set job path
-  set_tmp_path <- function(path) {
+  set_tmp_path <- function(path = Sys.getenv("SLURMR_TMP_PATH", getwd())) {
+
+    if (is.null(path))
+      path <- Sys.getenv("SLURMR_TMP_PATH", getwd())
 
     # Path normalization and assignment
     path <- normalizePath(path)
@@ -92,7 +33,7 @@ opts_slurmR <- (function() {
   # Function to get the path
   get_tmp_path <- function() {
     if (!length(OPTS_R$tmp_path))
-      OPTS_R$tmp_path <- getwd()
+      OPTS_R$tmp_path <- "" # getwd()
     OPTS_R$tmp_path
   }
 
@@ -108,7 +49,7 @@ opts_slurmR <- (function() {
       stop(
         "`name` must be a meaningful name. Cannot be \"\" (empty).",
         call. = FALSE
-        )
+      )
 
     fn <- sprintf("%s/%s", get_tmp_path(), name)
 
@@ -133,6 +74,8 @@ opts_slurmR <- (function() {
 
   attr(get_job_name, "desc") <- "Returns the current value of `job-name`."
 
+
+
   # Generalized set/get function -----------------------------------------------
   set_opts <- function(...) {
 
@@ -146,13 +89,13 @@ opts_slurmR <- (function() {
 
     Map(
       function(x., value.) {
-         if (!length(value.))
-            rm(list = x., envir = OPTS_SLURM)
-         else
-            assign(x=x., value=value., envir = OPTS_SLURM)
+        if (!length(value.))
+          rm(list = x., envir = OPTS_SLURM)
+        else
+          assign(x=x., value=value., envir = OPTS_SLURM)
       },
       x. = names(dots), value. = unname(dots)
-      )
+    )
 
     invisible()
 
@@ -198,6 +141,25 @@ opts_slurmR <- (function() {
   attr(get_opts_r, "desc")   <- "A generic function to retrieve options in R."
   attr(get_opts_job, "desc") <- "A generic function to retrieve options for the job (Slurm)."
 
+  get_preamble <- function() {
+    return(OPTS_PREAMBLE$dat)
+  }
+
+  set_preamble <- function(...) {
+    OPTS_PREAMBLE$dat <- c(OPTS_PREAMBLE$dat, unlist(list(...)))
+    invisible()
+  }
+
+  attr(set_preamble, "desc") <- paste(
+    "Sets \"preamble\" to the RScript call. For example, it could be used for loading",
+    "modules, setting env variables, etc., needed during the R session. Options are",
+    "passed as characters."
+  )
+  attr(get_preamble, "desc") <- paste(
+    "Returns the preamble, e.g., module loads, environment variable definitions, etc.,",
+    "that will be included in sbatch submissions."
+  )
+
   # Debugging and Verbose ------------------------------------------------------
 
   debug_on <- function() {
@@ -241,13 +203,15 @@ opts_slurmR <- (function() {
   # Final structure ------------------------------------------------------------
   structure(list2env(
     list(
-      set_tmp_path    = set_tmp_path,
-      get_tmp_path    = get_tmp_path,
+      set_tmp_path = set_tmp_path,
+      get_tmp_path = get_tmp_path,
       set_job_name = set_job_name,
       get_job_name = get_job_name,
       set_opts     = set_opts,
       get_opts_job = get_opts_job,
       get_opts_r   = get_opts_r,
+      set_preamble = set_preamble,
+      get_preamble = get_preamble,
       debug_on     = debug_on,
       debug_off    = debug_off,
       get_debug    = structure(
@@ -259,11 +223,103 @@ opts_slurmR <- (function() {
       get_cmd      = structure(
         function() OPTS_R$cmd,
         desc =  "If debug mode is active, then it returns `sh`, otherwise `sbatch`"
+      ),
+      reset        = structure(
+        function() {
+          .opts_slurmR()
+         opts_slurmR$set_tmp_path()
+          invisible()
+          },
+        desc = "Resets the options to the original state."
       )
     )
   ), class = "opts_slurmR")
 
-})()
+}
+
+
+#' Get and set default options for `sbatch` and `slurmR` internals
+#'
+#' Most of the functions in the `slurmR` package use `tmp_path` and `job-name`
+#' options to write and submit jobs to **Slurm**. These options have global
+#' defaults that are set and retrieved using `opts_slurmR`. These options
+#' also include SBATCH options and things to do before calling RScript,
+#' e.g., loading modules on an HPC cluster.
+#'
+#' Whatever the path specified on `tmp_path`, all nodes should have access to it.
+#' Moreover, it is recommended to use a path located in a high-performing drive.
+#' See for example [disk staging](https://en.wikipedia.org/w/index.php?title=Disk_staging&oldid=908353920).
+#'
+#' The `tmp_path` directory is only created at the time that one of the functions
+#' needs to I/O files. Job creation calls like [Slurm_EvalQ] and [Slurm_lapply]
+#' do such.
+#'
+#' The "preamble" options can be specified if, for example, the current cluster
+#' needs to load R, a compiler, or other programs via a `module` command.
+#'
+#' @details Current supported options are:
+#'
+#' Debugging mode
+#'
+#' - `debug_on : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$debug_on, "desc")}
+#'
+#' - `debug_off : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$debug_off, "desc")}
+#'
+#' - `get_debug : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_debug, "desc")}
+#'
+#' Verbose mode
+#'
+#' - `verbose_on : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$verbose_on, "desc")}
+#'
+#' - `verbose_off : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$verbose_off, "desc")}
+#'
+#' - `get_verbose : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_verbose, "desc")}
+#'
+#' Slurm options
+#'
+#' - `set_tmp_path : function (path, recursive = TRUE)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$set_tmp_path, "desc")}
+#'
+#' - `get_tmp_path : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_tmp_path, "desc")}
+#'
+#' - `set_job_name : function (path, check = TRUE, overwrite = TRUE)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$set_job_name, "desc")}.
+#'
+#' - `get_job_name : function (check = TRUE)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_job_name, "desc")}
+#'
+#' - `set_preamble : function (...)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$set_preamble, "desc")}
+#'
+#' - `get_preamble : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_preamble, "desc")}
+#'
+#'
+#' Other options
+#'
+#' - `get_cmd : function ()` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_cmd, "desc")}
+#'
+#' For general set/retrieve options
+#'
+#' - `set_opts : function (...)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$set_opts, "desc")}
+#'
+#' - `get_opts_job : function (...)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_opts_job, "desc")}
+#'
+#' - `get_opts_r : function (...)` \Sexpr[stage=build]{attr(slurmR::opts_slurmR$get_opts_r, "desc")}
+#'
+#'
+#' Nuke
+#'
+#' - While reloading the package should reset all the options, if needed, the user
+#'   can also use the function `opts_slurmR$reset()`.
+#'
+#' @examples
+#'
+#' # Common setup
+#' \dontrun{
+#' opts_slurmR$set_tmp_path("/staging/pdt/vegayon")
+#' opts_slurmR$set_job_name("simulations-1")
+#' opts_slurm$set_opts(partition="thomas", account="lc_pdt")
+#' opts_slurm$set_preamble("module load gcc")# if needed
+#' }
+#'
+#' @export
+opts_slurmR <- .opts_slurmR()
 
 #' Fill options for slurm using slurmr's defaults (if any)
 #' @noRd
@@ -288,6 +344,12 @@ print.opts_slurmR <- function(x, ...) {
 
   cat("\nOptions for sbatch (Slurm workflow):\n")
   options_printer(x$get_opts_job())
+  cat("\nPreamble:\n")
+  if (length(x$get_preamble())) {
+    cat(paste("  ", x$get_preamble(), collapse="\n"), "\n")
+  } else {
+    cat("  n/a\n")
+  }
   cat("\nOther options (R workflow):\n")
   options_printer(x$get_opts_r())
   cat("\nTo get and set options for Slurm jobs creation use (see ?opts_slurmR):\n\n")
