@@ -1,4 +1,13 @@
 #' A Lightweight Wrapper for 'Slurm'
+#'
+#' 'Slurm', Simple Linux Utility for Resource Management
+#' \url{https://slurm.schedmd.com/}, is a popular 'Linux' based software used to
+#' schedule jobs in 'HPC' (High Performance Computing) clusters. This R package
+#' provides a specialized lightweight wrapper of 'Slurm' with a syntax similar to
+#' that found in the 'parallel' R package. The package also includes a method for
+#' creating socket cluster objects spanning multiple nodes that can be used with
+#' the 'parallel' package.
+#'
 #' @details
 #' To cite slurmR in publications use:
 #'
@@ -29,7 +38,8 @@ NULL
 
 .onLoad <- function(libname, pkgname) {
 
-  opts_slurmR$set_tmp_path(getwd())
+  # It will set either the envir var SLURMR_TMP_PATH or getwd()
+  opts_slurmR$set_tmp_path()
 
 }
 
@@ -37,9 +47,10 @@ NULL
 .onAttach <- function(libname, pkgname) {
 
   packageStartupMessage(
-    "slurmR default option for `tmp_path` (used to store auxiliar files) set to:\n  ", getwd(),
+    "slurmR default option for `tmp_path` (used to store auxiliar files) set to:\n  ", opts_slurmR$get_tmp_path(),
     "\nYou can change this and checkout other slurmR options using: ",
     "?opts_slurmR, or you could just type \"opts_slurmR\" on the terminal."
   )
 
 }
+
